@@ -1,3 +1,5 @@
+from pathlib import Path
+
 import yaml
 import click
 import openai
@@ -16,7 +18,9 @@ from .constants import (
     DEFAULT_TEMPERATURE,
     DEFAULT_TOP_P,
     DEFAULT_FREQUENCY_PENALTY,
-    DEFAULT_PRESENCE_PENALTY, MAX_INPUT_TRIES
+    DEFAULT_PRESENCE_PENALTY, MAX_INPUT_TRIES, OPENAI_NUM_ANSWERS_MIN, OPENAI_MAX_TOKENS_MIN, OPENAI_TEMPERATURE_MIN,
+    OPENAI_TEMPERATURE_MAX, OPENAI_TOP_P_MIN, OPENAI_TOP_P_MAX, OPENAI_FREQUENCY_PENALTY_MIN,
+    OPENAI_FREQUENCY_PENALTY_MAX, OPENAI_PRESENCE_PENALTY_MIN, OPENAI_PRESENCE_PENALTY_MAX
 )
 
 
@@ -27,8 +31,11 @@ class AvailableModels(Enum):
     TEXT_DAVINCI_003 = auto()
 
     @classmethod
-    def as_list(cls) -> list:
-        return list(map(lambda c: c.name.replace("_", "-").lower(), cls))
+    def members_as_list(cls, openai_style: bool = True) -> list:
+        if openai_style:
+            return list(map(lambda c: c.name.replace("_", "-").lower(), cls))
+        else:
+            return list(map(lambda c: c.name, cls))
 
 
 @dataclass
@@ -42,9 +49,9 @@ class ConfigHelper:
     presence_penalty: float = DEFAULT_PRESENCE_PENALTY
 
     @classmethod
-    def from_file(cls) -> 'ConfigHelper':
-        if CONFIG_PATH.is_file():
-            with open(CONFIG_PATH, "r") as f:
+    def from_file(cls, config_path: Path = CONFIG_PATH) -> 'ConfigHelper':
+        if config_path.is_file():
+            with open(config_path, "r", encoding="utf8") as f:
                 config = yaml.safe_load(f)
                 return cls(**config)
         else:
@@ -52,12 +59,12 @@ class ConfigHelper:
                                     "Run 'askai config reset' to create a default config.", fg="red"))
             exit()
 
-    def input_model(self) -> None:
+    def input_model(self, max_input_tries: int = MAX_INPUT_TRIES) -> None:
         model = input("Choose model (1-4): ")
         num_of_tries = 1
 
         while not _is_int(model) or int(model) not in range(1, 5):
-            if num_of_tries >= MAX_INPUT_TRIES:
+            if num_of_tries >= max_input_tries:
                 click.echo(click.style("Too many invalid tries. Aborted!", fg="red"))
                 exit(1)
 
@@ -69,44 +76,84 @@ class ConfigHelper:
         click.echo(click.style(f"Model chosen: {self.model}", fg="green"))
         click.echo()
 
-    def input_num_answer(self) -> None:
-        self.num_answers = self._input_integer(default_value=DEFAULT_NUM_ANSWERS, predicate=lambda x: x > 0)
-
-    def input_max_token(self) -> None:
-        self.max_tokens = self._input_integer(default_value=DEFAULT_MAX_TOKENS, predicate=lambda x: x > 0)
-
-    def input_temperature(self) -> None:
-        self.temperature = self._input_float(default_value=DEFAULT_TEMPERATURE, predicate=lambda x: 0.0 <= x <= 1.0)
-
-    def input_top_p(self) -> None:
-        self.top_p = self._input_float(default_value=DEFAULT_TOP_P, predicate=lambda x: 0.0 <= x <= 1.0)
-
-    def input_frequency_penalty(self) -> None:
-        self.frequency_penalty = self._input_float(
-            default_value=DEFAULT_FREQUENCY_PENALTY,
-            predicate=lambda x: -2.0 <= x <= 2.0
+    def input_num_answer(self,
+                         default_value: int = DEFAULT_NUM_ANSWERS,
+                         min_value: int = OPENAI_NUM_ANSWERS_MIN,
+                         max_input_tries: int = MAX_INPUT_TRIES) -> None:
+        self.num_answers = self._input_integer(
+            default_value=default_value,
+            predicate=lambda x: x >= min_value,
+            max_input_tries=max_input_tries
         )
 
-    def input_presence_penalty(self) -> None:
+    def input_max_token(self,
+                        default_value: int = DEFAULT_MAX_TOKENS,
+                        min_value: int = OPENAI_MAX_TOKENS_MIN,
+                        max_input_tries: int = MAX_INPUT_TRIES) -> None:
+        self.max_tokens = self._input_integer(
+            default_value=default_value,
+            predicate=lambda x: x >= min_value,
+            max_input_tries=max_input_tries
+        )
+
+    def input_temperature(self,
+                          default_value: float = DEFAULT_TEMPERATURE,
+                          min_value: float = OPENAI_TEMPERATURE_MIN,
+                          max_value: float = OPENAI_TEMPERATURE_MAX,
+                          max_input_tries: int = MAX_INPUT_TRIES) -> None:
+        self.temperature = self._input_float(
+            default_value=default_value,
+            predicate=lambda x: min_value <= x <= max_value,
+            max_input_tries=max_input_tries
+        )
+
+    def input_top_p(self,
+                    default_value: float = DEFAULT_TOP_P,
+                    min_value: float = OPENAI_TOP_P_MIN,
+                    max_value: float = OPENAI_TOP_P_MAX,
+                    max_input_tries: int = MAX_INPUT_TRIES) -> None:
+        self.top_p = self._input_float(
+            default_value=default_value,
+            predicate=lambda x: min_value <= x <= max_value,
+            max_input_tries=max_input_tries
+        )
+
+    def input_frequency_penalty(self,
+                                default_value: float = DEFAULT_FREQUENCY_PENALTY,
+                                min_value: float = OPENAI_FREQUENCY_PENALTY_MIN,
+                                max_value: float = OPENAI_FREQUENCY_PENALTY_MAX,
+                                max_input_tries: int = MAX_INPUT_TRIES) -> None:
+        self.frequency_penalty = self._input_float(
+            default_value=default_value,
+            predicate=lambda x: min_value <= x <= max_value,
+            max_input_tries=max_input_tries
+        )
+
+    def input_presence_penalty(self,
+                               default_value: float = DEFAULT_PRESENCE_PENALTY,
+                               min_value: float = OPENAI_PRESENCE_PENALTY_MIN,
+                               max_value: float = OPENAI_PRESENCE_PENALTY_MAX,
+                               max_input_tries: int = MAX_INPUT_TRIES) -> None:
         self.presence_penalty = self._input_float(
-            default_value=DEFAULT_PRESENCE_PENALTY,
-            predicate=lambda x: -2.0 <= x <= 2.0
+            default_value=default_value,
+            predicate=lambda x: min_value <= x <= max_value,
+            max_input_tries=max_input_tries
         )
 
     def as_dict(self) -> dict:
         return asdict(self)
 
-    def update(self) -> None:
+    def update(self, config_path: Path = CONFIG_PATH) -> None:
         config = self.as_dict()
-        with open(CONFIG_PATH, "w") as f:
+        with open(config_path, "w", encoding="utf8") as f:
             yaml.dump(config, f)
 
         click.echo(click.style("Config updated successfully!", fg="green"))
 
     @staticmethod
-    def reset() -> None:
+    def reset(config_path: Path = CONFIG_PATH) -> None:
         config = ConfigHelper().as_dict()  # Create config with default values
-        with open(CONFIG_PATH, "w") as f:
+        with open(config_path, "w", encoding="utf8") as f:
             yaml.dump(config, f)
 
         click.echo("\nDefault config has been created with the following values:")
@@ -116,12 +163,12 @@ class ConfigHelper:
         click.echo("To change the config, please see: 'askai config --help'\n")
 
     @staticmethod
-    def show() -> None:
-        if not CONFIG_PATH.is_file():
+    def show(config_path: Path = CONFIG_PATH) -> None:
+        if not config_path.is_file():
             click.echo("No config exists. Please reset the config ('askai config reset') "
                        "or see 'askai config --help'.\n")
         else:
-            with open(CONFIG_PATH, "r") as f:
+            with open(config_path, "r", encoding="utf8") as f:
                 try:
                     config = yaml.safe_load(f)
                     for key, value in config.items():
@@ -131,8 +178,9 @@ class ConfigHelper:
 
     @staticmethod
     def _input_integer(default_value: int,
-                       predicate: Callable[[int], bool] = lambda x: True) -> int:
-        for _ in range(MAX_INPUT_TRIES):
+                       predicate: Callable[[int], bool] = lambda x: True,
+                       max_input_tries: int = MAX_INPUT_TRIES) -> int:
+        for _ in range(max_input_tries):
             input_value = input(f"Choose (press enter for default = {default_value}): ")
 
             if input_value == "":
@@ -156,8 +204,9 @@ class ConfigHelper:
 
     @staticmethod
     def _input_float(default_value: float,
-                     predicate: Callable[[float], bool] = lambda x: True) -> float:
-        for _ in range(MAX_INPUT_TRIES):
+                     predicate: Callable[[float], bool] = lambda x: True,
+                     max_input_tries: int = MAX_INPUT_TRIES) -> float:
+        for _ in range(max_input_tries):
             input_value = input(f"Choose (press enter for default = {default_value}): ")
 
             if input_value == "":
@@ -180,9 +229,9 @@ class ConfigHelper:
         exit(1)
 
 
-@dataclass
 class KeyHelper:
-    api_key: str = ""
+    def __init__(self):
+        self._api_key: str = ""
 
     def input(self) -> None:
         key = getpass("Enter API Key: ")
@@ -196,22 +245,26 @@ class KeyHelper:
             key = getpass("Enter API Key: ")
             num_tries += 1
 
-        self.api_key = key
+        self._api_key = key
 
-    def save(self) -> None:
-        API_KEY_PATH.parent.mkdir(parents=True, exist_ok=True)
-        API_KEY_PATH.write_text(self.api_key)
+    def save(self, api_key_path: Path = API_KEY_PATH) -> None:
+        api_key_path.parent.mkdir(parents=True, exist_ok=True)
+        api_key_path.write_text(self._api_key, encoding="utf8")
         click.echo(click.style("Your API key has been successfully added!", fg="green"))
 
     @staticmethod
-    def remove() -> None:
-        API_KEY_PATH.unlink()
-        click.echo(click.style("API key removed.", fg="green"))
+    def remove(api_key_path: Path = API_KEY_PATH) -> None:
+        try:
+            api_key_path.unlink()
+            click.echo(click.style("API key removed.", fg="green"))
+        except FileNotFoundError:
+            click.echo(click.style("No API key found.", fg="red"))
+            exit()
 
-    @classmethod
-    def from_file(cls) -> str:
-        if API_KEY_PATH.is_file():
-            with open(API_KEY_PATH, "r") as f:
+    @staticmethod
+    def from_file(api_key_path: Path = API_KEY_PATH) -> str:
+        if api_key_path.is_file():
+            with open(api_key_path, "r", encoding="utf8") as f:
                 api_key = f.read().strip()
             return api_key
         else:
@@ -352,7 +405,7 @@ class PrintHelper:
     @staticmethod
     def model() -> None:
         click.echo("   Different models have different capabilities.")
-        for idx, model_name in enumerate(AvailableModels.as_list()):
+        for idx, model_name in enumerate(AvailableModels.members_as_list(openai_style=True)):
             click.echo(f"   {idx+1}) {model_name}")
 
     @staticmethod
